// ---------------------------------------------------------------------------
// -      ATMEL Microcontroller Software Support  -   ROUSSET -
// ---------------------------------------------------------------------------
//  The software is delivered "AS IS" without warranty or condition of any
//  kind, either express, implied or statutory. This includes without
//  limitation any warranty or condition with respect to merchantability or
//  fitness for any particular purpose, or against the infringements of
//  intellectual property rights of others.
// ---------------------------------------------------------------------------
// - File Name           : eb40a.inc
// - Object              : Evaluation Board EB40A Definition File.
// -
// - 1.0 06/09/01 PFi    : Creation
// - 1.1 11/12/01 PFi    : Clean up
//
// - 1.1a 1.5.2006 Stefan : Modified for NavMe-Board and GNUARM
//                          Filename AT91R40008-NavMe.inc
// ---------------------------------------------------------------------------

// ------------------------------
// - Internal Memories Definition
// ------------------------------
.EQU	RAM_BASE_BOOT, 0x00300000	// RAM address before REMAP

// ------------------------------
// - External Memories Definition
// ------------------------------
// - Flash Memory: AT49BV322A
.EQU   FLASH_BASE, 0x01000000
.EQU   FLASH_SIZE, 0x400000      	//  4Mx8 read as 2Mx16

// - SDRAM Memory:
.EQU   EXT_SRAM_BASE, 0x03000000
.EQU   EXT_SRAM_SIZE, (8*1024*1024)
.EQU   EXT_SRAM_LIMIT, (EXT_SRAM_BASE + EXT_SRAM_SIZE)

// - If there is no external RAM memory...
// - Redefinition of External SRAM to Internal SRAM
// - EXT_SRAM_BASE   EQU     RAM_LIMIT
// - EXT_SRAM_SIZE   EQU     (RAM_SIZE-RAM_LIMIT)
// - EXT_SRAM_LIMIT  EQU     (RAM_BASE + RAM_SIZE)

// ---------------------------------------------------------------------------
// - EBI Initialization Data
// -------------------------
// - The EBI User Interface Image which is copied by the boot.
// - 66MHz master clock assumed.
// - That's hardware! Details in the Electrical Datasheet of the AT91 device.
// - EBI Base Address is added at the end for commodity in copy code.
// ---------------------------------------------------------------------------

.EQU   EBI_BASE,  0xFFE00000

.EQU   EBI_CSR_0, (FLASH_BASE    | 0x00003531)	// 0x01000000, 2 TDF, 16 Bit, 5 WS
.EQU   EBI_CSR_1, (EXT_SRAM_BASE | 0x00003525)	// 0x03000000, 2 TDF, 16 Bit, 2 WS
.EQU   EBI_CSR_2, 0x20000000  //  unused
.EQU   EBI_CSR_3, 0x30000000  //  unused
.EQU   EBI_CSR_4, 0x40000000  //  unused
.EQU   EBI_CSR_5, 0x50000000  //  unused
.EQU   EBI_CSR_6, 0x60000000  //  unused
.EQU   EBI_CSR_7, 0x70000000  //  unused

// ---------------------------------------------------------------------------
// AIC User Interface
// ---------------------------------------------------------------------------

.EQU   AIC_BASE,  0xFFFFF000

// ---------------------------------------------------------------------------
// Master Clock
// ---------------------------------------------------------------------------

.EQU MCK, 66000000

