//
// gcc-uart-polling example 
// 
// Target is AT91R40008 on NavMe-Board
// Toolchain is GNUARM 4.1.0
//
// 3.5.2006 v0.1 Stefan
// 

#include "testuart.h"

static void uart0Init(void)
{
  // Define RXD and TXD as peripheral
  PIO_PDR |=   (1<<P14_TXD0) | (1<<P15_RXD0);
  PIO_PER &= ~((1<<P14_TXD0) | (1<<P15_RXD0));

  // First, enable the clock
  // US0
  PS_PCER |= (1<<2);

  // Reset receiver and transmitter
  //       COM0->US_CR = AT91C_US_RSTRX | AT91C_US_RSTTX;

  // Reset Status Bits (PARE, FRAME, OVRE and RXBRK)
  //       COM0->US_CR = AT91C_US_RSTSTA;

  // Baudrate
  US0_BRGR = (66000000UL / 4800UL) / 16; 

  // async, 8 Bit, N, 1, normal mode, 
  US0_MR = 0 | (1<<CHRL1) | (1<<CHRL0) | (1<<PAR2);

  // Stop Peripheral Data Transfer
  US0_RCR = 0;
  US0_TCR = 0;

  // RX, TX enable
  US0_CR = 0 | (1<<RXEN) | (1<<TXEN);
}

static void uart1Init(void)
{
  // Define RXD and TXD as peripheral
  PIO_PDR |=   (1<<P21_TXD1) | (1<<P22_RXD1);
  PIO_PER &= ~((1<<P21_TXD1) | (1<<P22_RXD1));

  // First, enable the clock
  // US1
  PS_PCER |= (1<<3);

  // Reset receiver and transmitter
  //       COM0->US_CR = AT91C_US_RSTRX | AT91C_US_RSTTX;

  // Reset Status Bits (PARE, FRAME, OVRE and RXBRK)
  //       COM0->US_CR = AT91C_US_RSTSTA;

  // Baudrate
  US1_BRGR = (66000000UL / 4800UL) / 16; 

  // async, 8 Bit, N, 1, normal mode, 
  US1_MR = 0 | (1<<CHRL1) | (1<<CHRL0) | (1<<PAR2);

  // Stop Peripheral Data Transfer
  US1_RCR = 0;
  US1_TCR = 0;

  // RX, TX enable
  US1_CR = 0 | (1<<RXEN) | (1<<TXEN);
}

static void uart0Putch(unsigned char data)
{
  while(!(US0_CSR & ((1<<TXRDY)|(1<<TXEMPTY))));
  US0_THR = (unsigned long) data;
}

#if 0
// not used in this example
static unsigned char uart0Getch(void)
{
  unsigned char data = '+';
  while(!(US0_CSR & (1<<RXRDY)));
  data = (unsigned char) (US0_RHR & 0x000000FF);
  return data;
}
#endif

static unsigned char uart1Getch(void)
{
  unsigned char data = '+';
  while(!(US1_CSR & (1<<RXRDY)));
  data = (unsigned char) (US1_RHR & 0x000000FF);
  return data;
}

static void uart0PutString(const char *s)
{
	while(*s)
		uart0Putch(*s++);
}

int main(void)
{
  PIO_PER  |= (1<<16);		// ENABLE PIO CONTROL PIN16
  PIO_OER  |= (1<<16);		// ENABLE PIO OUTPUT
  PIO_SODR |= (1<<16);		// SET PIO OUTPUT

  uart0Init();
  uart1Init();

  uart0Putch('\f');

#ifdef RAM_RUN  
  uart0PutString("gcc-uart-polling (RAM_RUN)\n\r\n\r");
#endif 

#ifdef ROM_RUN
  uart0PutString("gcc-uart-polling (ROM_RUN)\n\r\n\r");
#endif 

  while(1) {
    unsigned char ch = '#';
    ch = uart1Getch();
    uart0Putch(ch);
  }

  return 0;	// never reached!
}
