/************************************************************************/
/*                                                                     	*/
/*  startup.S				                                        	*/
/*																	    */
/*  v0.1 - Stefan - 1st version for GNUARM & AT91R40008 on NavMe board	*/
/*                  Toolchain is GNUARM 4.1.0                          	*/
/*                  written with teachings from Atmel tutorials and		*/
/*                  from WinARM example code                            */
/*                                                                     	*/
/************************************************************************/

//
// Definition of memory layout
//
#include "AT91R40008-NavMe.inc"

    .text
    .code 32
    .align 2

// ###########################################
// Exception vectors before REMAP
// Running in ROM mapped at address 0x00000000
// ###########################################
	    .global _boot
	    .func   _boot
_boot:
        B InitReset    // Reset
undefvec:
        B undefvec     // Undefined Instruction
swivec:
        B swivec       // Software Interrupt
pabtvec:
        B pabtvec      // Prefetch Abort
dabtvec:
        B dabtvec      // Data Abort
rsvdvec:
        B rsvdvec      // Reserved
irqvec:
        B irqvec       // IRQ
fiqvec:
        B fiqvec       // FIQ

//
// EBI Initialization Table
//
// The External Bus Interface table is used to configure the memory controller
// and values depend on target. These values are defined in an include file
// corresponding to the target.
//
InitTableEBI:
	.word EBI_CSR_0     // Chip Select 0
    .word EBI_CSR_1     // Chip Select 1
    .word EBI_CSR_2     // Chip Select 2
    .word EBI_CSR_3     // Chip Select 3
    .word EBI_CSR_4     // Chip Select 4
	.word EBI_CSR_5     // Chip Select 5
	.word EBI_CSR_6     // Chip Select 6
	.word EBI_CSR_7     // Chip Select 7
	.word 1             // REMAP command
	.word 0             // 0 memory regions, standard read (CHANGED! 6 => 0)
PtEBIBase:
	.word EBI_BASE
//
// Reset handler before REMAP
// (linked at ROM at 0xXXXXXXXX)
//
InitReset:
	//
	// Speedup the boot sequence
	//
	// Speedup code execution by setting exact wait state on Chip Select 0
	// After reset, the EBI is not configured apart from the chip select 0.
	// The number of wait states on chip select 0 is 8.
	// Here, the chip select 0 configuration can be modified to speed up
	// the boot sequence with exact memory characteristics.
	//
		// load system EBI Base address and CSR0 init value
		ldr r0, PtEBIBase
		ldr r1, InitTableEBI
		str r1, [r0]

#if 0
	//
	// not modified for GNUARM
	//

	//
	// Peripherals that must be initialized before enabling interrupts.
	// At reset, some microcontrollers start with the slow clock 
	// oscillator (32.768kHz typically) to minimize the power required 
	// to start up the system.
	//
	// At this point, the PLL can be started by configuring the APMC to
	// run with the main oscillator to speed up the startup sequence.
	//

		//
		// Low level init
		//
		// ??? maybe a function in AT91 library
		//
		bl __low_level_init
#endif 

#if 0
	//
	// not tested for GNUARM
	//

	//
	// Advanced Interrupt Controller configuration
	// After reset, the AIC is not configured. The C-startup sequence
	// initializes the AIC by setting up the default interrupt vectors.
	// The default interrupt handler functions are defined in the AT91
	// libray and can be redefined in the application code.
	//
	
		// Load the AIC base address and the default handler addresses
		add r0, pc, #-8+.-AicData	// @ where to read values
									// (relative)
		ldmia r0, {r1-r4}
		// Setup the spurious vector
		str r4, [r1, #AIC_SPU]
		// Setup the default interrupt handler vectors
		str r2, [r1, #AIC_SVR]		// SVR[0] for FIQ
		add r1, r1, #AIC_SVR
		mov r0, #31					// counter
	LoopAic1:
		str r3, [r1, r0, LSL #2]	// SVRs for IRQs
		subs r0, r0, #1             // do not save FIQ
		bhi LoopAic1
		b EndInitAic
		
	AicData:
       	.word AIC_BASE
	PtDefaultHandler:
      	.word at91_default_fiq_handler
        .word at91_default_irq_handler
        .word at91_spurious_handler

	// v0.1 1st version loop until reset
	// 		should be improved   
	.global at91_default_fiq_handler
	at91_default_fiq_handler:
    	b     .             
	.global at91_default_irq_handler
	at91_default_irq_handler:
    	b     .                         
	.global at91_spurious_handler
    at91_spurious_handler:
    	b     .           
		
	EndInitAic:
#endif

	//
	// Setup exception vectors in internal RAM before REMAP
	//
	// The exception vectors must be copied into internal RAM in order to
	// guarantee that the core is provided valid vectors during the remap
	// operation.
	//
		b SetupRamVectors
	VectorTable:
#if 1
		ldr pc, _SoftReset
		ldr pc, _UndefHandler
		ldr pc, _SWIHandler
		ldr pc, _PrefetchAbortHandler
        ldr pc, _DataAbortHandler
        nop
        ldr pc, [pc, #-0xF20]      // IRQ read the AIC
        ldr pc, [pc, #-0xF20]      // FIQ read the AIC
	_SoftReset:
        .word SoftReset
    _UndefHandler:
		.word UndefHandler
	_SWIHandler:
        .word SWIHandler
	_PrefetchAbortHandler:
		.word PrefetchAbortHandler
	_DataAbortHandler:
		.word DataAbortHandler
#else
		ldr pc, [pc, #&18]         // SoftReset
        ldr pc, [pc, #&18]         // UndefHandler
        ldr pc, [pc, #&18]         // SWIHandler
        ldr pc, [pc, #&18]         // PrefetchAbortHandler
        ldr pc, [pc, #&18]         // DataAbortHandler
        nop
        ldr pc, [pc, #-0xF20]      // IRQ read the AIC
        ldr pc, [pc, #-0xF20]      // FIQ read the AIC
        .word SoftReset
        .word UndefHandler
        .word SWIHandler
        .word PrefetchAbortHandler
        .word DataAbortHandler
#endif

        // Vectoring Execution funktion run at absolute address
	SoftReset:
        b SoftReset
	UndefHandler:
        b UndefHandler
	SWIHandler:
		b SWIHandler
	PrefetchAbortHandler:
		b PrefetchAbortHandler
	DataAbortHandler:
		b DataAbortHandler

	SetupRamVectors:
		mov r8, #RAM_BASE_BOOT    // @ of the hard vector in RAM
        add r9, pc, #-(8+.-VectorTable) // @ where to read values
										// (relative)
		ldmia r9!, {r0-r7}        // read 8 vectors
		stmia r8!, {r0-r7}        // store them on RAM
        ldmia r9!, {r0-r4}        // read 5 absolute handler adresses
        stmia r8!, {r0-r4}        // store them on RAM

		//
        // Memory controller initialization
        //
        	// copy the image of the memory controller
        	sub r10, pc, #(8+.-InitTableEBI)        // get address of the chip
		
			// select register image
          	ldr r12, PtInitRemap                    // get the real jump address (after remap)
          	
			// copy chip select register image to memory controller and command remap
          	ldmia r10!, {r0-r9,r11}                 // load the complete image with the remap command
          	stmia r11!, {r0-r9}                     // store the complete image with the remap command
		
		//
		// After reset before remap command, the internal RAM is
		// mapped at address 0x00300000 and the memory connected to
		// the chip select 0 is mapped at address 0x0.
		//
		// After remap command, the internal RAM is then mapped at
		// address 0x0 allowing the ARM exception vectors to be
		// modified by the software.
		//
		// The ARM processor pipeline assures that the instruction
		// mov pc,r12 will be read before remap is executed.
		//
		// After remap, the next instruction is fetched in internal
		// RAM and the instruction mov pc,r12 is executed to perform
		// jump in ROM at the previously loaded address in r12 and
		// break the pipeline.
		//
		
          	// jump to ROM at its new address
          	mov pc, r12		// jump and break the pipeline
	PtInitRemap:
		.word InitRemap		// Address where to jump after REMAP
    .size _boot, . - _boot
    .endfunc

// ###################################################################
// From here the code is executed from its link address, ie. 0x0100000
// ###################################################################
	.global _start, start, _mainCRTStartup
	.func   _start
_start:
start:
_mainCRTStartup:
InitRemap:
	//
	// Stack size definitions
	//
	.set IRQ_STACK_SIZE, (4*8*4)	// 32 words		??? 3*8 => 4*8
	.set FIQ_STACK_SIZE, (3*4)	// 3 words
	.set ABT_STACK_SIZE, (1*4)	// 1 words
	.set UND_STACK_SIZE, (1*4)	// 1 words
	.set SVC_STACK_SIZE, (1*4)	// ??? words

	// Standard definitions of Mode bits and Interrupt (I & F) flags in PSRs
	// ???
    .set  ARM_MODE_USR, 0x10        // User Mode
    .set  ARM_MODE_FIQ, 0x11        // FIQ Mode
    .set  ARM_MODE_IRQ, 0x12        // IRQ Mode
    .set  ARM_MODE_SVC, 0x13        // Supervisor Mode
    .set  ARM_MODE_ABT, 0x17        // Abort Mode
    .set  ARM_MODE_UND, 0x1B        // Undefined Mode
    /* .set  ARM_MODE_SYS, 0x1F        // System Mode		??? */

#if 0
	//
	// not tested for GNUARM
	// we do not use two different stacks in 
	// internal/external RAM configuration
	//

	//
	// Top of stack definition
	//
	.equ TOP_EXCEPTION_STACK, RAM_LIMIT
	.equ TOP_APPLICATION_STACK, EXT_RAM_LIMIT	// 	???
#endif 

	//
	// Setup stack for each mode
	//
	// Fast interrupt, interrupt, abort, undefined and supervisor stacks
	// are located at the top of internal memory in order to speed up the
	// exception handling context.
	//
	// User stack is located at the top of the external memory. ???
	//
	// The initialization code initializes the stack pointer registers.
	//
	.global _stack			// top of stack
	ldr r0, =_stack			// ldr r0, =TOP_EXCEPTION_STACK

	//
	// The initialization code can now enable interrupts if necessary.
	// This is the earliest point that it is safe to enable interrupts.
	//
	.equ  I_BIT, 0x80       // when I bit is set, IRQ is disabled
    .equ  F_BIT, 0x40		// when F bit is set, FIQ is disabled

	// Setup Fast Interrupt mode and set FIQ Mode Stack
	msr CPSR_c, #ARM_MODE_FIQ|I_BIT|F_BIT
	mov r13, r0
	sub r0, r0, #FIQ_STACK_SIZE

	// Setup Interrupt mode and set IRQ Mode Stack
	msr CPSR_c, #ARM_MODE_IRQ|I_BIT|F_BIT
	mov r13, r0
	sub r0, r0, #IRQ_STACK_SIZE

	// Setup Abort mode and set Abort Mode Stack
	msr CPSR_c, #ARM_MODE_ABT|I_BIT|F_BIT
	mov r13, r0
	sub r0, r0, #ABT_STACK_SIZE

	// Setup Undefined Instruction mode and set
	// Undefined Instruction Mode Stack
	msr CPSR_c, #ARM_MODE_UND|I_BIT|F_BIT
	mov r13, r0
	sub r0, r0, #UND_STACK_SIZE

	// Setup Supervisor mode and set Supervisor Mode Stack
	msr CPSR_c, #ARM_MODE_SVC|I_BIT|F_BIT
	mov r13, r0
	sub r0, r0, #SVC_STACK_SIZE

#if 0
	//
	// not tested for GNUARM
	//

	//
	// At this stage, the processor is still in supervisor mode, if the
	// application runs in User mode, change to user mode and initialize
	// the user mode stack.
	//
	msr CPSR_c, #ARM_MODE_USER			// set User mode
	ldr r13, =TOP_APPLICATION_STACK		// Init stack User
#endif

	//
	// The next task is to initialize the data memory by entering a loop
	// that writes zeroes into allocations used for data storage.
	// In C language, any non-initialized variable is supposed to contain
	// zero as an initial value.
	//

		//
		// Initialize C variables
		//
		// The initial values for any initialized variables must be
		// copied from ROM to RAM and all other variables must be
		// initialized to zero.
		//

			// Clear BSS (zero init)
			.global __bss_start             	// -> .bss area in RAM
    		.global __bss_end__             	// end of .bss area

			ldr r1,=__bss_end__		// ldr r1, =__end_bss
			ldr r3,=__bss_start		// ldr r3, =__begin_bss
			mov r2, #0
		LoopZI:
			cmp r3, r1
			strcc r2, [r3], #4
			bcc LoopZI

#if 0
	//
	// not modified for GNUARM
	//
			// Clear SBSS (Zero Init)
			.weak __begin_bss
			.weak __end_bss

			ldr r1, =__end_sbss
			ldr r3, =__begin_sbss
			mov r2, #0
		LoopZI2:
			cmp r3, r1
			strcc r2, [r3], #4
			bcc LoopZI2
#endif

#ifdef ROM_RUN
			// Copy initialized DATA from FLASH to RAM
    		.global _etext         	// -> .data initial values in ROM
    		.global _data          	// -> .data area in RAM
    		.global _edata         	// end of .data area

			ldr r0, =_etext			// __begin_romdata
			ldr r1, =_data			// __begin_data
			ldr r3, =_edata			// __end_data
		LoopRW:
			cmp r1, r3
			ldrcc r2, [r0], #4
			strcc r2, [r1], #4
			bcc LoopRW
#endif /* ROM_RUN */

	// ##################################################
	// Branch on C code main function (with interworking)
	// ##################################################
	.global main                    // int main(void)
	ldr r0, =main			// __main
	bx r0

	.size   _start, . - _start
	.endfunc

//
// The function main() should be a closed loop and should not return.
//
    	.global _reset, reset, exit, abort
    	.func   _reset
_reset:
reset:
exit:
abort:
    	b     .                         // loop until reset

    	.size _reset, . - _reset
	.endfunc
.end

